/**
 * Collect and update data from a price/discount/value table. This data can for example be used
 * to get compare data for the *unique* validator from *ep.validate*.
 *
 *
 * ### Example
 * Binds the presentationUiPriceTableData()-widget to a price list table. Each time the validation is triggered by
 * the *textedit*-widget, data from the presentationUiPriceTableData-widget gets updated. The updated data gets
 * handed over to the validation module.
 *
 *
 * HTML:
 *
 * <table class="ep-bulkprices-table ep-bulkprices-table_1 ContentList">
 *   <thead>
 *     <tr>
 *       <td class="Checkbox">
 *         <input type="checkbox" class="visually-hidden ep-js" data-js=".uiInput()" name="selector" value="Checkbox" onclick="OnCheckAllInTable($(this).closest('table'), 'DeleteQuantity', this.checked);" /> </td>
 *       <td class="ep-bulkprices-table-quantity-row">Menge</td>
 *       <td>Einzelpreis</td>
 *       <td>Rabatt</td>
 *     </tr>
 *   </thead>
 *   <tbody>
 *     <tr class="ep-bulkprices-existingentry-row">
 *       <td class="Checkbox">
 *         <input type="checkbox" class="RunButtonTrigger ep-js" data-js=".uiInput()" name="DeleteQuantity" value="1" /> </td>
 *       <td class="ep-bulkprices-clickcell "> <span data-item='{"name": "Quantity", "value": 1}' class="ep-js" data-js=".uiTextedit({name: 'Quantity', validate: {required: true, type: 'unique', additionalValidator: 'number', format: 'n0', region: $.i18n.settings.region, min: 1, getCompareData: function(){ return ep('.ep-bulkprices-table_1').presentationUiPriceTableData('getIndexFilteredDataByName', 0, 'Quantity'); }}, setAttr: {name: true, value: '1'}, buildInstantly: true, validateBeforeClose: function(opts) { return false;}, onAfterClose: function(input){ setTimeout(function() {input.closest('.ep-bulkprices-clickcell').one('click.openTextedit', function(evt) { (evt.target.nodeName.toUpperCase() === 'TD') && $(evt.target).find('span').uiTextedit('open'); });}, 300) }})">1</span> </td>
 *       <td class="ep-bulkprices-clickcell"> <span data-item='{"name": "Price", "value": "1,00 €"}' class="ep-js" data-js=".uiTextedit({name: 'Price', validate: {required: false, type: 'number', format: 'c2', region: $.i18n.settings.region, currency: $.i18n.settings.currency}, setAttr: {name: true, value: '1,00 €'}, buildInstantly: true, formatOnWriteBack: function(input, evt, error){if(input.val() !== ''){input.val($.i18n.formatNumber($.i18n.parseNumber(input.val()), 'c2', {region: $.i18n.settings.region, currency: $.i18n.settings.currency}));}}, onAfterClose: function(input){ setTimeout(function() {input.closest('.ep-bulkprices-clickcell').one('click.openTextedit', function(evt) { (evt.target.nodeName.toUpperCase() === 'TD') && $(evt.target).find('span').uiTextedit('open'); });}, 300) }})">1,00 €</span></td>
 *       <td class="ep-bulkprices-clickcell"> <span data-item='{"name": "Discount", "value": 1}' class="ep-js" data-js=".uiTextedit({name: 'Discount', validate: {required: false, max: 100, type: 'number', format: 'n1', region: $.i18n.settings.region}, setAttr: {name: true, value: '1'}, buildInstantly: true, onBeforeOpen: function(input){input.val() !== '' && input.val(parseFloat(input.val()));}, formatContentOnAfterWriteBack: function(widget) {widget.content=widget.uiInput.val() === '' ? '' : $.i18n.formatNumber($.i18n.parseNumber(widget.uiInput.val()), 'p1', {region: $.i18n.settings.region});}, formatOnWriteBack: function(input, evt, error){input.val()!=='' && input.val($.i18n.formatNumber($.i18n.parseNumber(input.val()), 'n1', {region: $.i18n.settings.region}));}, onAfterClose: function(input){ setTimeout(function() {input.closest('.ep-bulkprices-clickcell').one('click.openTextedit', function(evt) { (evt.target.nodeName.toUpperCase() === 'TD') && $(evt.target).find('span').uiTextedit('open'); });}, 300) }})">1 %</span> </td>
 *     </tr>
 *     <tr class="NewEntry ep-bulkprices-newentry-error">
 *       <td class="Checkbox"></td>
 *       <td class="DialogError">
 *         <input data-item='{"name": "Quantity", "value": 5 }' type="text" name="NewQuantity" class="ep-bulkprices-newentry-field ep-js ep-uiInput ep-uiInput-text ep-uiValidate" data-js=".uiValidate({required: true, type: 'unique', additionalValidator: 'number', format: 'n0', region: $.i18n.settings.region, min: 1, getCompareData: function(){ return ep('.ep-bulkprices-table_1').presentationUiPriceTableData('getIndexFilteredDataByName', 3, 'Quantity'); }})" value="5" size="4"> </td>
 *       <td>
 *         <div class="ep-flex ep-flex-align-center ">
 *           <input data-item='{"name": "Price", "value": "5,00 €"}' type="text" name="NewPrice" class="ep-bulkprices-newentry-field ep-js ep-uiInput ep-uiInput-text ep-uiValidate" data-js=".uiValidate({required: false, type: 'number', format: 'c2', region: $.i18n.settings.region, currency: $.i18n.settings.currency})" value="5">
 *           <span class="ep-bulkprices-newentry-currency">€</span>
 *         </div>
 *       </td>
 *       <td>
 *         <div class="ep-flex ep-flex-align-center ">
 *           <input data-item='{"name": "Discount", "value": "5"}' type="text" name="NewDiscount" class="ep-bulkprices-newentry-field ep-js ep-uiInput ep-uiInput-text ep-uiValidate" data-js=".uiValidate({required: false, max: 100, type: 'number', format: 'n1', region: $.i18n.settings.region})" value="5">
 *         </div>
 *       </td>
 *     </tr>
 *     <tr class="ep-bulkprices-notsaved">
 *       <td class="Checkbox"> </td>
 *       <td class="ep-bulkprices-clickcell"> <span class="ep-js" data-js="">6</span>
 *         <input type="text" class="ep-uiInput ep-uiInput-text ep-uiValidate ep-uiTextedit-wrap" name="NewQuantity" style="display: none;"> </td>
 *       <td class="ep-bulkprices-clickcell"> <span class="ep-js" data-js="">6,00 €</span>
 *         <input type="text" class="ep-uiInput ep-uiInput-text ep-uiValidate ep-uiTextedit-wrap" name="NewPrice" style="display: none;">
 *       <td class="ep-bulkprices-clickcell"> <span class="ep-js" data-js="">6,0&nbsp;%</span>
 *         <input type="text" class="ep-uiInput ep-uiInput-text ep-uiValidate ep-uiTextedit-wrap" name="NewDiscount" style="display: none;"> </td>
 *     </tr>
 *     <tr id="ep-bulkprices-newentry_1" class="ep-bulkprices-newentry">
 *       <td class="Checkbox"> </td>
 *       <td>
 *         <input type="text" name="NewQuantity" class="ep-bulkprices-newentry-field ep-js ep-uiInput ep-uiInput-text ep-uiValidate" data-js="" value="" size="4"> </td>
 *       <td>
 *         <div class="ep-flex ep-flex-align-center">
 *           <input type="text" name="NewPrice" class="ep-bulkprices-newentry-field ep-js ep-uiInput ep-uiInput-text ep-uiValidate ui-changed" data-js="" value="">
 *           <span class="ep-bulkprices-newentry-currency">€</span>
 *         </div>
 *       </td>
 *       <td>
 *         <div class="ep-flex ep-flex-align-center">
 *           <input type="text" name="NewDiscount" class="ep-bulkprices-newentry-field ep-js ep-uiInput ep-uiInput-text ep-uiValidate" data-js="" value="">
 *           <span class="ep-bulkprices-newentry">&nbsp;%</span>
 *         </div>
 *       </td>
 *     </tr>
 *   </tbody>
 * </table>
 *
 *
 * JavaScript:
 *
 *    <script>
 *       require([
 *           'jquery',
 *           'ep',
 *
 *           'de_epages/presentation/ui/pricetabledata',
 *           '$ready!'
 *       ], function($, ep) {
 *           $('.ep-bulkprices-table_1').presentationUiPriceTableData({
 *               rows: {
 *                   existing: '.ep-bulkprices-existingentry-row',
 *                   error: '.ep-bulkprices-newentry-error',
 *                   notsaved: '.ep-bulkprices-notsaved'
 *               }
 *           });
 *       });
 *   </script>
 *
 *
 *
 * @class jQuery.ui.presentationUiPriceTableData
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses jQuery.ui.widget
 * @since 7.58.0
 */

/**
 * @cfg {Object} [rows] object that contains the class names of the rows which contain the data
 * @cfg {Object} [constData] object that contains constant data (e.g. {'Quantity': [1,2], 'Discount': [7, 6]})
 */

/**
 * See `jQuery.ui.presentationUiPriceTableData` for details.
 *
 * @param {Object} [rows] object that contains the class names of the rows which contain the data
 * @param {Object} [constData] object that contains constant data (e.g. {'Quantity': [1,2], 'Discount': [7, 6]})
 *
 *
 * @method presentationUiPriceTableData
 * @member jQuery
 *
 * @since 7.58.0
 */

/*
 * @copyright       © Copyright 2006-2021, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/presentation/ui/pricetabledata
 */

define("de_epages/presentation/ui/pricetabledata", [
    "jquery",
    "ep",
    "de_epages",

    "jquery/ui/widget"
], function($, ep, de_epages) {

    $.widget('ui.presentationUiPriceTableData', {
    //  values,
    //  selector

        options: {
            rows: {
                existing: '.ep-bulkprices-existingentry-row',
                error: '.ep-bulkprices-newentry-error',
                notsaved: '.ep-bulkprices-notsaved'
            },
            constData: {}
        },

        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.presentationUiPriceTableData
         *
         * @since 7.58.0
         */
        _create: function() {
            var self = this,
                o = self.options,
                // extract rows selector from rows option
                selector = Object.keys(o.rows).map(function(key) {
                    return 'tbody ' + o.rows[key] + ',';
                }).join("");

            // remove last *,*
            self.selector = selector.substring(0, selector.length - 1);
        },

        /**
         * Method to update data inside the table,
         * e.g. data = {'Quantity': [1, 2, 3], 'Price': [4, 5, 6]}
         *
         * @member jQuery.ui.presentationUiPriceTableData
         *
         * @since 7.58.0
         */
        updateData: function() {
            var self = this,
                elem = self.element,
                o = self.options,
                values;

            values = $.extend({}, o.constData);

            // iterate over existing rows
            elem.find(
                self.selector
            ).each(function(index) {
                var $row = $(this),
                    items = $row.find('input').filter(function() {
                      return $(this).data('item') !== undefined;
                    });

                // iterate over relevant data items containing the name/value pairs
                items.each(function() {
                    var $item = $(this),
                        content = $item.data('item');

                    if (content.name) {
                        // if no array for *name* property exists yet => create it
                        if (values[content.name] === undefined) {
                            values[content.name] = [];
                        }
                        // add input value to array
                        values[content.name] = values[content.name].concat($item.val());
                    }
                });
            });

            // save to widget
            self.values = values;
        },

        /**
         * Get data corresponding to a name/key,
         * e.g. name = 'Quantity' =>  {data: [1, 2, 3]}.
         *
         * @member jQuery.ui.presentationUiPriceTableData
         *
         * @since 7.58.0
         */
        getDataByName: function(name, parseFn) {
            var self = this,
                result;

            // update data first
            self.updateData();

            result = {
                data: self.values[name] || []
            };

            // if a function to parse/format values is present => execute it
            if (parseFn !== undefined && typeof parseFn === "function") {
                result.data = result.data.map(function(val){return parseFn(val)});
            }

            return result;
        },

        /**
         * Get data corresponding to a name/key and the value
         * at index-position get sliced and returned as value,
         * e.g. name='Quantity', index=1 and 'Quantity' = [1, 2, 3] => {data: [1, 3], value: 2}
         *
         * @member jQuery.ui.presentationUiPriceTableData
         *
         * @since 7.58.0
         */
        getIndexFilteredDataByName: function(index, name, parseFn) {
            var self = this,
                values,
                result;

            // update data first
            self.updateData();

            values = self.values[name] || [];

            result = {
                // filter data by index (value at index position gets removed)
                data: values.filter(function(item, itemIndex) {
                    return index !== itemIndex;
                }),
                // value at index position is returned
                value: values.filter(function(item, itemIndex) {
                    return index === itemIndex;
                })[0]
            };

            // if a function to parse/format values is present => execute it
            if (parseFn !== undefined && typeof parseFn === "function") {
                result.data = result.data.map(function(val){return parseFn(val)});
                result.value = parseFn(result.value);
            }

            return result;
        },

        destroy: function() {
            var self = this;

            this._superApply(arguments);
        }
    });

    return de_epages;
});
