/*globals define, window*/

define('de_epages/product/inc/variationslideshowactivator', [
    'jquery',
    '$dict!de_epages/product/inc/variationslideshowactivator',

    'ep/ui/tooltip'
], function ($, dict) {
    'use strict';

    /*
     * @dictionary       de_epages.product.inc.variationslideshowactivator
     *
     * @translation      {DisabledImageSettingsTooltip}
     */

    var translations = {
            tInfoText: dict.translate('DisabledImageSettingsTooltip')
        };

    return function (options) {
        var defaults = {
                target: '',
                slideshowArrays: {}
            },
            o = $.extend({}, defaults, options),
            $batchActionRadio = o.target.find('input[type=radio]'),
            titleElem = o.target.find('div[title]')[0],
            batchActionState = $batchActionRadio.prop('checked'),

            inputs,
            activeInputs,
            activeIds,
            tooltip,
            titleText,

            // check if all slideshow string of all selected variations are equal
            checkValidity = function (activeInputs) {
                // get array containing the ids of all selected variations
                var activeIds = activeInputs.map(function(index, checkbox) {
                        return checkbox.value;
                    }).get(),
                    firstSlideshowString,
                    isNotEqual;

                if (activeIds.length === 0) {
                    return false;
                }

                // build slideshow string for the first selected variation
                firstSlideshowString = o.slideshowArrays[activeIds[0]].join(';');

                // check if all slideshow strings are equal
                isNotEqual = activeIds.some(function(id) {
                    return o.slideshowArrays[id].join(';') !== firstSlideshowString;
                });

                if (isNotEqual) {
                    return false;
                }

                return true;
            },

            // creates a tooltip
            initTooltip = function() {
                tooltip = $('<div>' + translations.tInfoText + '</div>')
                        .uiTooltip({
                            interactive: false,
                            event: 'hover',
                            context: o.target,
                            orientation: 'bottom',
                            hideDelay: 200,
                            oldhandling: true,
                            additionalClass: '',
                            offsetAdjust: [0, 7],
                        });
            };

        // if no target node exists
        if (!(o.target.length && o.target[0].nodeType)) {
            console.warn('variationslideshowactivator: option "target" is no jquery-element.');
            return;
        }

        // register a handler (listen to the batchactions.js event; fires if a checkbox was selected)
        $(document).on('checkCompleted:batchaction', function(evt, data) {
            // get active inputs (handed over by batchaction.js)
            activeInputs = data.activeInputs;

            if (activeInputs.length > 0) {
                if (checkValidity(activeInputs)) {
                    o.target.removeClass('BatchActionDisabled');
                    $batchActionRadio.prop('checked', batchActionState);

                    if (tooltip) {
                        titleElem.title = titleText;
                        tooltip.uiTooltip('option', 'disabled', true);
                    }
                } else {
                    o.target.addClass('BatchActionDisabled');
                    batchActionState = $batchActionRadio.prop('checked');

                    if (!tooltip) {
                        initTooltip();
                        titleText = titleElem.title;
                    } else {
                        tooltip.uiTooltip('option', 'disabled', false);
                    }
                    titleElem.title = "";
                }
            }
        });
    };
});

